#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <cmath>

using namespace std;

// Headers


class Color;
class Buffer;
class Quaternion;
class Vector3;
class Ray;
class Intersection;
class Light;
class Sphere;
class Box;
class Renderable;

double ERROR_RANGE = 0.01;
double TRACE_GIVE_UP = 0.1;
double MAX_TRACE_NUMBER = 9;

double PIXEL_X = 200;
double PIXEL_Y = 200;
double FOCUS = 100;
int ANTI_A = 2;

class Color {
public:
	double r;
	double g;
	double b;

	Color();
	Color(double vr , double vg, double vb);
	Color clone();
	Color add(Color color);
	Color sub(Color color);
	Color mul(Color color);
	Color scale(double f);
	double light();
	void write(ofstream * file, int colormax);
	double min();
};

class Buffer {
public:
	int mx;
	int my;
	int aa;
	Color *** buffer;
	Buffer(int x, int y, int a);
	~Buffer();
	void write(ofstream * file, int colormax);
	void output();
};

class Quaternion {
public:
	double x;
	double y;
	double z;
	double w;
	Quaternion(double a, double b, double c, double d);
	Quaternion grassman(Quaternion q);
	Quaternion conjugate();
};

class Vector3 {
public:
	double x;
	double y;
	double z;
	Vector3();
	Vector3(double a, double b, double c);
	Vector3 clone();
	Vector3 add(Vector3 v);
	Vector3 sub(Vector3 v);
	Vector3 mul(Vector3 v);
	Vector3 div(Vector3 v);
	Vector3 scale(double a);
	double dot(Vector3 v);
	double length();
	Vector3 cross(Vector3 v);
	Vector3 unit();
	double delta_radius(Vector3 v);
	Vector3 rotate_around(Vector3 axis, double radius);
	Vector3 rotate_to(Vector3 target, double radius);
};

class Ray {
public:
	Vector3 loc;
	Vector3 dir;
	Ray();
	Ray(Vector3 a, Vector3 b);
	Ray clone();
	Intersection ** all_intersections();
	Intersection first_intersection();
};

class Intersection {
public:
	Ray ray;
	double lamda;
	Ray normal;
	int going_in;
	Renderable * renderable;
	Intersection();
	Intersection(Ray r, double l, Ray n, Renderable * rd, int g);
	Intersection clone();
};

class Light {
public:
	Vector3 location;
	Color color;
	Light(Vector3 l, Color c);
	Color receive(Vector3 l);
};

class Renderable {
public:
	Color color_rfl;
	Color color_rfr;
	double rfr_index;

	virtual Intersection ** intersection_normal(Ray ray) { return 0; };
};

class Sphere : public Renderable {
public:
	Vector3 location;
	double radius;

	Sphere(Vector3 l, double r, Color c, double i);
	double * intersection_lamda(Ray ray);
	Intersection ** intersection_normal(Ray ray);
};

class Box : public Renderable {
public:
	Vector3 minimum;
	Vector3 maximum;

	Box(Vector3 a, Vector3 b, Color c, double i);
	Intersection ** intersection_normal(Ray ray);
};

// Headers end.



// Global Variable
Renderable ** renderables;
int reneerable_num;
Light ** lights;
int light_num;

// class color {
Color::Color() {
	r = 0;
	g = 0;
	b = 0;
}
Color::Color(double vr , double vg, double vb) {
	r = vr ;
	g = vg;
	b = vb;
}
Color Color::clone() {
	return Color(r, g, b);
}
Color Color::add(Color color) {
	return Color(r + color.r, g + color.g, b + color.b);
}
Color Color::sub(Color color) {
	return Color(r - color.r, g - color.g, b - color.b);
}
Color Color::mul(Color color) {
	return Color(r * color.r, g * color.g, b * color.b);
}
Color Color::scale(double f) {
	return Color(r * f, g * f, b * f);
}
void Color::write(ofstream * file, int colormax) {
	int wr = floor(r * colormax);
	int wg = floor(g * colormax);
	int wb = floor(b * colormax);

	if (wr < 0) {
		wr = 0;
	}
	if (wg < 0) {
		wg = 0;
	}
	if (wb < 0) {
		wb = 0;
	}
	if (wr >= colormax) {
		wr = colormax - 1;
	}
	if (wg >= colormax) {
		wg = colormax - 1;
	}
	if (wb >= colormax) {
		wb = colormax - 1;
	}
	*file << " " << wr << " " << wg << " " << wb << " " << "\n";
}
double Color::light() {
	return r + g + b;
}
double Color::min() {
	if (r < g) {
		if (r < b) {
			return r;
		}
		else {
			return b;
		}
	}
	else {
		if (g < b) {
			return g;
		}
		else {
			return b;
		}
	}
}
// }

// class Buffer {
Buffer::Buffer(int x, int y, int a) {
	mx = x;
	my = y;
	aa = a;
	int cx = 0;
	int cy = 0;

	buffer = (Color ***)malloc(mx * aa * sizeof(Color **));
	while (cx < mx * aa) {
		buffer[cx] = (Color **)malloc(my * aa * sizeof(Color *));
		cy = 0;
		while (cy < my * aa) {
			buffer[cx][cy] = new Color();
			cy++;
		}
		cx++;
	}
}

Buffer::~Buffer() {
	int cx = 0;
	int cy = 0;
	while (cx < mx * aa) {
		cy = 0;
		while (cy < my * aa) {
			delete buffer[cx][cy];
			cy++;
		}
		cx++;
	}
}

void Buffer::write(ofstream * file, int colormax) {
	*file << "P3\n";
	*file << mx << " " << my << "\n";
	*file << colormax << "\n";

	int cx = 0;
	int cy = my;

	int cax;
	int cay;
	Color temp;
	while (cy > 0) {
		cx = 0;
		cy--;
		while (cx < mx) {

			temp = Color(0, 0, 0);
			cax = 0;
			while (cax < aa) {
				cay = 0;
				while (cay < aa) {
					temp = temp.add(*(buffer[cx * aa + cax][cy * aa + cay]));
					cay++;
				}
				cax++;
			}
			temp = temp.scale(1 / double(aa * aa));
			temp.write(file, colormax);

			cx++;
		}
	}
}

void Buffer::output() {
	ofstream * myfile = new ofstream;
	myfile->open("example.ppm");
	write(myfile, 64);
	myfile->close();
	delete myfile;
}
// };

// class Quaternion {
Quaternion::Quaternion(double a, double b, double c, double d) {
	x = a;
	y = b;
	z = c;
	w = d;
}
Quaternion Quaternion::grassman(Quaternion q) {
	double nx = (x * q.x) - (y * q.y) - (z * q.z) - (w * q.w);
	double ny = (x * q.y) + (y * q.x) + (z * q.w) - (w * q.z);
	double nz = (x * q.z) - (y * q.w) + (z * q.x) + (w * q.y);
	double nw = (x * q.w) + (y * q.z) - (z * q.y) + (w * q.x);
	return Quaternion(nx, ny, nz, nw);
}
Quaternion Quaternion::conjugate() {
	return Quaternion(x, -y, -z, -w);
}
// };

// class Vector3 {
Vector3::Vector3() {
	x = 0;
	y = 0;
	z = 0;
}
Vector3::Vector3(double a, double b, double c) {
	x = a;
	y = b;
	z = c;
}
Vector3 Vector3::clone() { return Vector3(x, y, z); }
Vector3 Vector3::add(Vector3 v) { return Vector3(x + v.x, y + v.y, z + v.z); }
Vector3 Vector3::sub(Vector3 v) { return Vector3(x - v.x, y - v.y, z - v.z); }
Vector3 Vector3::mul(Vector3 v) { return Vector3(x * v.x, y * v.y, z * v.z); }
Vector3 Vector3::div(Vector3 v) { return Vector3(x / v.x, y / v.y, z / v.z); }
Vector3 Vector3::scale(double a) { return Vector3(x * a, y * a, z * a); }
double Vector3::dot(Vector3 v) { return (x * v.x) + (y * v.y) + (z * v.z); }
double Vector3::length() { return pow(dot(clone()), 0.5); }
Vector3 Vector3::cross(Vector3 v) {
	double nx = y * v.z - z * v.y;
	double ny = z * v.x - x * v.z;
	double nz = x * v.y - y * v.x;
	return Vector3(nx, ny, nz);
}
Vector3 Vector3::unit() {
	double len = length();
	if (len == 0) {
		return Vector3(0, 0, 0);
	}
	return clone().scale(1 / len);
}
double Vector3::delta_radius(Vector3 v) {
	double u = dot(v);
	double d = length() * v.length();
	if (d == 0) {
		return 0;
	}
	return acos(u / d);
}
Vector3 Vector3::rotate_around(Vector3 axis, double radius) {
	axis = axis.unit();
	Quaternion q = Quaternion(cos(radius / 2), sin(radius / 2) * axis.x, sin(radius / 2) * axis.y, sin(radius / 2) * axis.z);
	Quaternion p = q.conjugate();
	Quaternion a = Quaternion(0, x, y, z);
	Quaternion b = q.grassman(a).grassman(p);
	return Vector3(b.y, b.z, b.w);
}
Vector3 Vector3::rotate_to(Vector3 target, double radius) {
	/*
	double delta = delta_radius(target);
	if (delta <= radius) {
	return target.clone();
	}
	*/
	Vector3 axis = cross(target).unit();
	return rotate_around(axis, radius);
}
// };

// class Ray {
Ray::Ray() {
	loc = Vector3();
	dir = Vector3();
}
Ray::Ray(Vector3 a, Vector3 b) {
	loc = a;
	dir = b.unit();
}
Ray Ray::clone() {
	return Ray(loc, dir);
}
Intersection ** Ray::all_intersections() {
	Intersection ** ans = (Intersection **)malloc(sizeof(Intersection*) * ((reneerable_num * 2) + 1));
	int i = 0;
	while (i < (reneerable_num * 2) + 1) {
		ans[i] = 0;
		i++;
	}
	Intersection ** intersections;
	int j = 0;
	i = 0;
	while (j < reneerable_num) {
		intersections = renderables[j]->intersection_normal(clone());
		if (intersections) {
			ans[i] = intersections[0];
			i++;
			ans[i] = intersections[1];
			i++;
			free(intersections);
		}
		j++;
	}
	return ans;
}
Intersection Ray::first_intersection() {
	Intersection ** all = all_intersections();
	int i = 0;
	int record = -1;
	while ((all[i]) && (i < reneerable_num * 2)) {
		if (((record < 0) || (all[record]->lamda > all[i]->lamda)) && (all[i]->lamda > ERROR_RANGE)) {
			record = i;
		}
		i++;
	}
	Intersection t = Intersection();
	t.lamda = -1;
	if (record >= 0) {
		t = all[record]->clone();
	}
	i = 0;
	while (all[i]) {
		delete all[i];
		i++;
	}
	free(all);
	return t;
}
// };

// class Intersection {
Intersection::Intersection() {
	ray = Ray();
	lamda = 0;
	normal = Ray();
}
Intersection::Intersection(Ray r, double l, Ray n, Renderable * rd, int g) {
	ray = r;
	lamda = l;
	normal = n;
	renderable = rd;
	going_in = g;
}
Intersection Intersection::clone() {
	Intersection t = Intersection(ray, lamda, normal, renderable, going_in);
	return t;
};
// };

// class Light {
Light::Light(Vector3 l, Color c) {
	location = l;
	color = c;
}
Color Light::receive(Vector3 l) {
	Ray ray = Ray(l, location.sub(l));
	Intersection intersection = ray.first_intersection();
	if (intersection.lamda < 0) {
		return color;
	}
	else {
		double distance = l.sub(location).length();
		if (intersection.lamda > distance) {
			return color;
		}
		else {
			return Color();
		}
	}
}
// };

// class Sphere {
Sphere::Sphere(Vector3 l, double r, Color c, double i) {
	location = l;
	radius = r;
	color_rfl = c;
	color_rfr = Color(1, 1, 1).sub(color_rfl);
	rfr_index = i;
}
double * Sphere::intersection_lamda(Ray ray) {
	double a = ray.dir.dot(ray.dir);
	double b = 2 * (ray.dir.dot(ray.loc) - ray.dir.dot(location));
	double c = (ray.loc.dot(ray.loc) + location.dot(location) - (2 * ray.loc.dot(location)) - (radius * radius));
	double delta = (b * b) - (4 * a * c);
	if ((delta < 0) || (a == 0)) {
		return 0;
	}
	double v1 = ((-b) + pow(delta, 0.5)) / (2 * a);
	double v2 = ((-b) - pow(delta, 0.5)) / (2 * a);
	double * ans = (double *)malloc(2 * sizeof(double));
	ans[0] = v1;
	ans[1] = v2;
	return ans;
}
Intersection ** Sphere::intersection_normal(Ray ray) {
	double * lamdas = intersection_lamda(ray);
	if (!lamdas) {
		return 0;
	}
	Intersection ** ans = (Intersection **)malloc(2 * sizeof(Intersection *));
	Ray normal;
	Vector3 intersection_loc;

	intersection_loc = ray.loc.add(ray.dir.scale(lamdas[0]));
	normal = Ray(intersection_loc, intersection_loc.sub(location));
	ans[0] = new Intersection(ray, lamdas[0], normal, this, ray.dir.dot(normal.dir) < 0);

	intersection_loc = ray.loc.add(ray.dir.scale(lamdas[1]));
	normal = Ray(intersection_loc, intersection_loc.sub(location));
	ans[1] = new Intersection(ray, lamdas[1], normal, this, ray.dir.dot(normal.dir) < 0);

	free(lamdas);
	return ans;
}
// };

// class Box {
Box::Box(Vector3 a, Vector3 b, Color c, double i) {
	minimum = a;
	maximum = b;
	color_rfl = c;
	color_rfr = Color(1, 1, 1).sub(color_rfl);
	rfr_index = i;
}
Intersection ** Box::intersection_normal(Ray ray) {
	double lamda;
	Intersection ** ans = (Intersection **)malloc(2 * sizeof(Intersection *));
	Ray normal;
	int i = 0;

	// minimum z
	lamda = (minimum.z - ray.loc.z) / ray.dir.z;
	normal = Ray(ray.loc.add(ray.dir.scale(lamda)), Vector3(0, 0, -1));
	if ((normal.loc.x > minimum.x) && (normal.loc.x < maximum.x) && (normal.loc.y > minimum.y) && (normal.loc.y < maximum.y)) {
		if ((i == 0) || ((i == 1) && (abs(lamda - ans[0]->lamda) > ERROR_RANGE))) {
			ans[i] = new Intersection(ray, lamda, normal, this, ray.loc.z < minimum.z);
			i++;
		}
	}

	// maximum z
	lamda = (maximum.z - ray.loc.z) / ray.dir.z;
	normal = Ray(ray.loc.add(ray.dir.scale(lamda)), Vector3(0, 0, 1));
	if ((normal.loc.x > minimum.x) && (normal.loc.x < maximum.x) && (normal.loc.y > minimum.y) && (normal.loc.y < maximum.y)) {
		if ((i == 0) || ((i == 1) && (abs(lamda - ans[0]->lamda) > ERROR_RANGE))) {
			ans[i] = new Intersection(ray, lamda, normal, this, ray.loc.z > maximum.z);
			i++;
		}
	}

	// minimum y
	lamda = (minimum.y - ray.loc.y) / ray.dir.y;
	normal = Ray(ray.loc.add(ray.dir.scale(lamda)), Vector3(0, -1, 0));
	if ((normal.loc.x > minimum.x) && (normal.loc.x < maximum.x) && (normal.loc.z > minimum.z) && (normal.loc.z < maximum.z)) {
		if ((i == 0) || ((i == 1) && (abs(lamda - ans[0]->lamda) > ERROR_RANGE))) {
			ans[i] = new Intersection(ray, lamda, normal, this, ray.loc.y < minimum.y);
			i++;
		}
	}

	// maximum y
	lamda = (maximum.y - ray.loc.y) / ray.dir.y;
	normal = Ray(ray.loc.add(ray.dir.scale(lamda)), Vector3(0, 1, 0));
	if ((normal.loc.x > minimum.x) && (normal.loc.x < maximum.x) && (normal.loc.z > minimum.z) && (normal.loc.z < maximum.z)) {
		if ((i == 0) || ((i == 1) && (abs(lamda - ans[0]->lamda) > ERROR_RANGE))) {
			ans[i] = new Intersection(ray, lamda, normal, this, ray.loc.y > maximum.y);
			i++;
		}
	}

	// minimum x
	lamda = (minimum.x - ray.loc.x) / ray.dir.x;
	normal = Ray(ray.loc.add(ray.dir.scale(lamda)), Vector3(-1, 0, 0));
	if ((normal.loc.y > minimum.y) && (normal.loc.y < maximum.y) && (normal.loc.z > minimum.z) && (normal.loc.z < maximum.z)) {
		if ((i == 0) || ((i == 1) && (abs(lamda - ans[0]->lamda) > ERROR_RANGE))) {
			ans[i] = new Intersection(ray, lamda, normal, this, ray.loc.x < minimum.x);
			i++;
		}
	}

	// maximum x
	lamda = (maximum.x - ray.loc.x) / ray.dir.x;
	normal = Ray(ray.loc.add(ray.dir.scale(lamda)), Vector3(1, 0, 0));
	if ((normal.loc.y > minimum.y) && (normal.loc.y < maximum.y) && (normal.loc.z > minimum.z) && (normal.loc.z < maximum.z)) {
		if ((i == 0) || ((i == 1) && (abs(lamda - ans[0]->lamda) > ERROR_RANGE))) {
			ans[i] = new Intersection(ray, lamda, normal, this, ray.loc.x > maximum.x);
			i++;
		}
	}

	if (i == 0) {
		free(ans);
		return 0;
	}

	if (i == 1) {
		delete(ans[0]);
		free(ans);
		return 0;
	}

	if (i == 2) {
		return ans;
	}
}
// };
Color trace(Ray ray, int bounce, Color significance) {
	if ((bounce < 0) || (significance.min() < TRACE_GIVE_UP)) {
		return Color(0, 0, 0);
	}
	Intersection intersection = ray.first_intersection();

	if (intersection.lamda < 0) {
		// Does not intersect with anything.
		return Color(0, 0, 0);
	}

	Color ans = Color();
	Ray rfl_ray;
	Ray rfr_ray;
	int i = 0;

	rfl_ray = Ray(intersection.normal.loc, ray.dir.scale(-1).rotate_to(intersection.normal.dir, 2 * ray.dir.scale(-1).delta_radius(intersection.normal.dir)));

	// Considering about reflection of light directly.
	double rfl_ratio;
	while (i < light_num) {
		// Use a silly way to simulate reflection of direct point light.
		rfl_ratio = cos(rfl_ray.dir.delta_radius(lights[i]->location.sub(rfl_ray.loc)));
		ans = ans.add(intersection.renderable->color_rfl.mul(lights[i]->receive(intersection.normal.loc)).scale(rfl_ratio));
		i++;
	}

	// Considering about reflection from environment. Using mirror reflection.
	ans = ans.add(intersection.renderable->color_rfl.mul(trace(rfl_ray, bounce - 1, significance.mul(intersection.renderable->color_rfl))));

	// Considering about refraction from environment. Using mirror reflection.
	double rfr_index = intersection.renderable->rfr_index;
	if (rfr_index > 0) {
		Vector3 normal_vec = intersection.normal.dir;
		if (!intersection.going_in) {
			rfr_index = 1 / rfr_index;
			normal_vec = normal_vec.scale(-1);
		}
		double theta_t = abs(asin(sin(ray.dir.scale(-1).delta_radius(normal_vec)) / rfr_index));
		// rfr_ray = Ray(intersection.normal.loc, normal_vec.cross(normal_vec.cross(ray.dir)).scale(sin(theta_t)).sub(normal_vec.scale(cos(theta_t))));
		rfr_ray = Ray(intersection.normal.loc, ray.dir.rotate_to(normal_vec.scale(-1), abs(ray.dir.delta_radius(normal_vec.scale(-1))) - theta_t));

		// printf("dir_x: %f %f\n", rfr_ray.dir.x, ray.dir.x);
		ans = ans.add(intersection.renderable->color_rfr.mul(trace(rfr_ray, bounce - 1, significance.mul(intersection.renderable->color_rfr))));
	}

	return ans;
}


void stop() {
	while (1) {
	}
}

// Direction is -z
void trace_array(Buffer * buffer, double focus, Vector3 origin) {
	int cx;
	int cy;
	Vector3 dir;
	Ray ray;
	Color c;
	cx = 0;
	while (cx < (buffer->mx) * (buffer->aa)) {
		cy = 0;
		while (cy < (buffer->my) * (buffer->aa)) {
			dir = Vector3((cx - (((double)((buffer->mx) * (buffer->aa))) / 2)) / (buffer->aa), (cy - (((double)((buffer->my) * (buffer->aa))) / 2)) / (buffer->aa), -focus);
			ray = Ray(origin, dir);
			c = trace(ray, MAX_TRACE_NUMBER, Color(1, 1, 1));
			buffer->buffer[cx][cy]->r = c.r;
			buffer->buffer[cx][cy]->g = c.g;
			buffer->buffer[cx][cy]->b = c.b;
			/*
			double lightness = buffer->buffer[cx][cy]->light();
			if (lightness > 0) {
			printf("x:%3d y:%3d\n col: %f, %f, %f\n", cx, cy, buffer->buffer[cx][cy]->r, buffer->buffer[cx][cy]->g, buffer->buffer[cx][cy]->b);
			}
			*/
			cy++;
		}
		printf("%d / %d\n", cx, (buffer->mx) * (buffer->aa));
		cx++;
	}
}

void test() {
	/*
	reneerable_num = 4;
	renderables = (Renderable **)malloc(reneerable_num * sizeof(Renderable *));
	renderables[0] = new Box(Vector3(-200, -200, -300), Vector3(200, 200, 100), Color(0.5, 0.5, 0.5), 0);
	renderables[1] = new Sphere(Vector3(-150, -150, -250), 40, Color(1, 0, 0), 0);
	renderables[2] = new Sphere(Vector3(150, -150, -250), 40, Color(0, 1, 0), 0);
	renderables[3] = new Sphere(Vector3(-150, 150, -250), 40, Color(0, 0, 1), 0);

	light_num = 3;
	lights = (Light **)malloc(light_num * sizeof(Light *));
	lights[0] = new Light(Vector3(0, 100, 0), Color(1, 0, 0));
	lights[1] = new Light(Vector3(100, 0, 0), Color(0, 1, 0));
	lights[2] = new Light(Vector3(0, 0, 75), Color(0, 0, 1));
	*/



	reneerable_num = 2;
	renderables = (Renderable **)malloc(reneerable_num * sizeof(Renderable *));
	renderables[0] = new Box(Vector3(-200, -200, -300), Vector3(200, 200, 100), Color(0.5, 0.5, 0.5), 0);
	renderables[1] = new Sphere(Vector3(0, 0, -100), 40, Color(0.1, 0.1, 0.1), 1.05);

	light_num = 3;
	lights = (Light **)malloc(light_num * sizeof(Light *));
	lights[0] = new Light(Vector3(0, 100, 0), Color(1, 0, 0));
	lights[1] = new Light(Vector3(100, 0, 0), Color(0, 1, 0));
	lights[2] = new Light(Vector3(-100, -100, 0), Color(0, 0, 1));

	Buffer * buffer = new Buffer(PIXEL_X, PIXEL_Y, ANTI_A);
	trace_array(buffer, FOCUS, Vector3(0, 0, 0));
	buffer->output();
	printf("Done.\n");
	// stop();
}

void do_file() {
	float tf0;
	float tf1;
	float tf2;
	float tf3;
	float tf4;
	float tf5;
	float tf6;
	float tf7;
	float tf8;
	float tf9;

	reneerable_num = 0;
	light_num = 0;

	ifstream file;
	file.open("data.txt");
	char c = 1;
	while (c >= 0) {
		c = -1;
		file >> c;
		if (c == 'I') {
			file >> PIXEL_X >> PIXEL_Y;
		}
		if (c == 'P') {
			file >> tf0;
			FOCUS = 1.0f / tf0;
		}
		if (c == 'M') {
			file >> MAX_TRACE_NUMBER;
		}
		if (c == 'B') {
			file >> tf0 >> tf1 >> tf2 >> tf3 >> tf4 >> tf5 >> tf6 >> tf7 >> tf8 >> tf9;
			reneerable_num = reneerable_num + 1;
		}
		if (c == 'S') {
			file >> tf0 >> tf1 >> tf2 >> tf3 >> tf4 >> tf5 >> tf6 >> tf7;
			reneerable_num = reneerable_num + 1;
		}
		if (c == 'L') {
			file >> tf0 >> tf1 >> tf2 >> tf3 >> tf4 >> tf5;
			light_num = light_num + 1;
		}
	}
	file.close();

	renderables = (Renderable **)malloc(reneerable_num * sizeof(Renderable *));
	lights = (Light **)malloc(light_num * sizeof(Light *));
	int renderable_index = 0;
	int light_index = 0;

	file.open("data.txt");
	c = 1;
	while (c >= 0) {
		c = -1;
		file >> c;
		if (c == 'I') {
			file >> PIXEL_X >> PIXEL_Y;
			printf("pixel: %d %d\n", PIXEL_X, PIXEL_Y);
		}
		if (c == 'P') {
			file >> tf0;
			FOCUS = 1.0f / tf0;
			printf("focus: %f\n", FOCUS);
		}
		if (c == 'M') {
			file >> MAX_TRACE_NUMBER;
			printf("max bounce: %d\n", MAX_TRACE_NUMBER);
		}
		if (c == 'B') {
			file >> tf0 >> tf1 >> tf2 >> tf3 >> tf4 >> tf5 >> tf6 >> tf7 >> tf8 >> tf9;
			printf("Box %d: <%f %f %f> <%f %f %f> (%f %f %f) %f\n", renderable_index, tf0, tf1, tf2, tf3, tf4, tf5, tf6, tf7, tf8, tf9);
			renderables[renderable_index] = new Box(Vector3(tf0, tf1, tf2), Vector3(tf3, tf4, tf5), Color(tf6, tf7, tf8), tf9);
			renderable_index = renderable_index + 1;
		}
		if (c == 'S') {
			file >> tf0 >> tf1 >> tf2 >> tf3 >> tf4 >> tf5 >> tf6 >> tf7;
			printf("Sphere %d: <%f %f %f> %f (%f %f %f) %f\n", renderable_index, tf0, tf1, tf2, tf3, tf4, tf5, tf6, tf7);
			renderables[renderable_index] = new Sphere(Vector3(tf0, tf1, tf2), tf3, Color(tf4, tf5, tf6), tf7);
			renderable_index = renderable_index + 1;
		}
		if (c == 'L') {
			file >> tf0 >> tf1 >> tf2 >> tf3 >> tf4 >> tf5;
			printf("Light%d : <%f %f %f> (%f %f %f)\n", light_index, tf0, tf1, tf2, tf3, tf4, tf5);
			lights[light_index] = new Light(Vector3(tf0, tf1, tf2), Color(tf3, tf4, tf5));
			light_index = light_index + 1;
		}
	}
	file.close();

	Buffer * buffer = new Buffer(PIXEL_X, PIXEL_Y, ANTI_A);
	printf("reneerable num: %d\n", reneerable_num);
	printf("light num: %d\n", light_num);
	printf("Trace start.\n");
	trace_array(buffer, FOCUS, Vector3(0, 0, 0));
	buffer->output();
	printf("Done.\n");
}

int main() {
	do_file();
}